/**************************************************************************
 **                                                                      **
 ** Copyright (C) 2011-2026 Lukas Spies                                  **
 ** Contact: https://photoqt.org                                         **
 **                                                                      **
 ** This file is part of PhotoQt.                                        **
 **                                                                      **
 ** PhotoQt is free software: you can redistribute it and/or modify      **
 ** it under the terms of the GNU General Public License as published by **
 ** the Free Software Foundation, either version 2 of the License, or    **
 ** (at your option) any later version.                                  **
 **                                                                      **
 ** PhotoQt is distributed in the hope that it will be useful,           **
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of       **
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        **
 ** GNU General Public License for more details.                         **
 **                                                                      **
 ** You should have received a copy of the GNU General Public License    **
 ** along with PhotoQt. If not, see <http://www.gnu.org/licenses/>.      **
 **                                                                      **
 **************************************************************************/
#pragma once

#include <QObject>
#include <QQmlEngine>
#include <scripts/pqc_scriptscontextmenu.h>

/*************************************************************/
/*************************************************************/
//
//      NOTE: This singleton is a wrapper for the C++ class
//            This class here can ONLY be used from QML!
//
/*************************************************************/
/*************************************************************/

class PQCScriptsContextMenuQML : public QObject {

    Q_OBJECT
    QML_ELEMENT
    QML_SINGLETON
    QML_NAMED_ELEMENT(PQCScriptsContextMenu)

public:
    PQCScriptsContextMenuQML() {
        connect(&PQCScriptsContextMenu::get(), &PQCScriptsContextMenu::customEntriesChanged, this, &PQCScriptsContextMenuQML::customEntriesChanged);
    }

    Q_INVOKABLE QVariantList getEntries() {
        return PQCScriptsContextMenu::get().getEntries();
    }
    Q_INVOKABLE void setEntries(QVariantList entries) {
        PQCScriptsContextMenu::get().setEntries(entries);
    }

    Q_INVOKABLE QVariantList detectSystemEntries() {
        return PQCScriptsContextMenu::get().detectSystemEntries();
    }

    Q_INVOKABLE void closeDatabase() {
        PQCScriptsContextMenu::get().closeDatabase();
    }

Q_SIGNALS:
    void customEntriesChanged();

};
